/*
 * @(#)virii.c
 *
 * Copyright 2000, Aaron Ardiri (mailto:aaron@ardiri.com)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation;  either version 2, or (at your option)
 * any version.
 *
 * This program is distributed in the hope that it will be useful,  but
 * WITHOUT  ANY  WARRANTY;   without  even   the  implied  warranty  of 
 * MERCHANTABILITY  or FITNESS FOR A  PARTICULAR  PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You  should have  received a  copy of the GNU General Public License
 * along with this program;  if not,  please write to the Free Software 
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * REVISIONS:
 * ----------
 *
 * 28-Aug-2000 Aaron Ardiri Creation
 * 29-Aug-2000 Aaron Ardiri Updated to GNU GPL licensing.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

int
main(int argc, char *argv[])
{
  int result = 0;

  printf("Liberty Trojan Checker for Palm Computing Platform\n");
  printf("  Copyright 2000 Aaron Ardiri\n\n");

  // called correctly?
  if (argc > 1) {

    unsigned short i;

    // process all data files passed in
    for (i=1; i<argc; i++) {

/*
 * virii: application signature
 */

#define LENGTH 90
      unsigned char  virii_length      = LENGTH;
      unsigned char  virii_signature[] = { 
                                           0x48, 0x6E, 0xFF, 0xDC, // 4
                                           0x48, 0x6E, 0xFF, 0xDA, // 8
					   0x42, 0x27, 0x42, 0xA7, // 12
					   0x2F, 0x3C, 0x61, 0x70, // 16
					   0x70, 0x6C, 0x76, 0xE0, // 20
					   0xD6, 0x8E, 0x2F, 0x03, // 24
					   0x1F, 0x3C, 0x00, 0x01, // 28
					   0x4E, 0x4F, 0xA0, 0x78, // 32
					   0x4F, 0xEF, 0x00, 0x18, // 36
					   0x4A, 0x40, 0x66, 0x2E, // 40
					   0x2F, 0x2E, 0xFF, 0xDC, // 44
					   0x3F, 0x2E, 0xFF, 0xDA, // 48
					   0x4E, 0x4F, 0xA0, 0x42, // 52
					   0x48, 0x6E, 0xFF, 0xDC, // 56
					   0x48, 0x6E, 0xFF, 0xDA, // 60
					   0x42, 0x27, 0x42, 0xA7, // 64
					   0x2F, 0x3C, 0x61, 0x70, // 68
					   0x70, 0x6C, 0x2F, 0x03, // 72
					   0x42, 0x27, 0x4E, 0x4F, // 76
					   0xA0, 0x78, 0x4F, 0xEF, // 80
					   0x00, 0x1E, 0x4A, 0x40, // 84
					   0x67, 0xD2, 0x4E, 0x4F, // 88
					   0xA0, 0x8C              // 90
                                         };

      unsigned char  *buffer = NULL; 
      unsigned int    buflen = 0;
      FILE           *inFile = NULL;

      // open the "rom" file
      inFile = fopen(argv[i],"rb");
     
      // does it exist?
      if (inFile != NULL) {

        // lets check the size
        fseek(inFile, 0L, SEEK_END); buflen = ftell(inFile); rewind(inFile);

        // allocate memory for the big read
	buffer = (unsigned char *)malloc(buflen);

	// did everything go ok?
	if (buffer != NULL) {

          unsigned long j;
	  int           virii;

          // read the "file" into memory
          fread(buffer,1,buflen,inFile); 

          // lets look for the "virii" signature
	  virii = 0; j = 0;
	  while ((!virii) && (j < buflen)) {

            // lets try and identify the "first" byte in the sequence
	    if (buffer[j] == virii_signature[0]) {

              unsigned long k;

              // ok.. lets look for the "whole" thing
	      k = 0;
	      while (
                     (k < virii_length) &&         // only search XX bytes
		     ((j+k) < buflen) &&           // dont overrun the buffer 
                     (buffer[j+k] == virii_signature[k])  // keep looking
                    ) {
                k++;
	      }

	      // was it a virus?
	      virii = (k == virii_length);
	    }

            j++;
	  }

          // did we detect something?
  	  if (virii) 
            printf("WARNING: VIRII detected - [%s]\n", argv[i]);
          else
            printf("OK!    : program clean  - [%s]\n", argv[i]);

          // free the memory allocated
	  free(buffer);
	}

        // close input file
        fclose(inFile);
      }
      else
        printf("ERROR: [%s] FILE NOT FOUND\n", argv[i]);
    }
    printf("\n");
  }
  else {
    printf("\nThis program is free software; you may redistribute it under\n"
           "terms of the GNU General Public License. This program has\n" 
	   "absolutely no warranty, you use it AS IS at your own risk.\n\n");
    printf("USAGE:\n");
    printf("  virii { file1 file2 ... fileN }\n\n");

    result = 1;
  }

  return result;
}
